using System.Collections.Generic;
using System.Linq;
using Luski.net.Enums;
using Luski.net.Structures.Main;
using Luski.net.Structures.Public;

namespace Luski.net;

public class API
{
    public MainServer MainServer { get; internal set; }
    internal List<PublicServer> InternalServers { get; } = new();
    public IReadOnlyList<PublicServer> LoadedServers => InternalServers.AsReadOnly();
    
    public PublicServer GetPublicServer(string Domain, string Version = "v1")
    {
        IEnumerable<PublicServer> isl = InternalServers.Where(a => (a.Domain == Domain && a.ApiVersion == Version));
        if (isl.Any()) return isl.First();
        PublicServer s = new()
        {
            Domain = Domain,
            ApiVersion = Version,
            Storage = new(Domain),
            EncryptionHandler = new(Domain),
            ServerType = ServerType.Public
        };
        InternalServers.Add(s);
        return s;
    }
    
    public MainServer GetMainServer(string Domain, string Version = "v1")
    {
        MainServer = new()
        {
            Domain = Domain,
            ApiVersion = Version,
            Storage = new(Domain),
            EncryptionHandler = new(Domain),
            ServerType = ServerType.Main
        };
        return MainServer;
    }
}